@@ -9,13 +9,20 @@ import os
 import random
 import time
 import warnings
+import json
 from json import JSONDecodeError
-from typing import Dict, Any, Union, Literal
+from typing import Dict, Any, Union, Literal, Optional
+import re
 
 import httpx
 
 from .s3_reader import S3BufferedReader, bytes_generator
-from .s3_types import Boto3ConnectionSettings, DuckDbConnectionSettings, PolarsConnectionSettings, S3Object
+from .s3_types import (
+    Boto3ConnectionSettings,
+    DuckDbConnectionSettings,
+    PolarsConnectionSettings,
+    S3Object,
+)
 
 _client: "Windmill | None" = None
 
@@ -26,7 +33,11 @@ JobStatus = Literal["RUNNING", "WAITING", "COMPLETED"]
 
 class Windmill:
     def __init__(self, base_url=None, token=None, workspace=None, verify=True):
-        base = base_url or os.environ.get("BASE_INTERNAL_URL")
+        base = (
+            base_url
+            or os.environ.get("BASE_INTERNAL_URL")
+            or os.environ.get("WM_BASE_URL")
+        )
 
         self.base_url = f"{base}/api"
         self.token = token or os.environ.get("WM_TOKEN")
@@ -39,7 +50,29 @@ class Windmill:
         self.workspace = workspace or os.environ.get("WM_WORKSPACE")
         self.path = os.environ.get("WM_JOB_PATH")
 
-        assert self.workspace, f"workspace required as an argument or as WM_WORKSPACE environment variable"
+        self.mocked_api = self.get_mocked_api()
+
+        assert self.workspace, (
+            f"workspace required as an argument or as WM_WORKSPACE environment variable"
+        )
+
+    def get_mocked_api(self) -> Optional[dict]:
+        mocked_path = os.environ.get("WM_MOCKED_API_FILE")
+        if not mocked_path:
+            return None
+        logger.info("Using mocked API from %s", mocked_path)
+        mocked_api = {"variables": {}, "resources": {}}
+        try:
+            with open(mocked_path, "r") as f:
+                incoming_mocked_api = json.load(f)
+            mocked_api = {**mocked_api, **incoming_mocked_api}
+        except Exception as e:
+            logger.warning(
+                "Error parsing mocked API file at path %s Using empty mocked API.",
+                mocked_path,
+            )
+            logger.debug(e)
+        return mocked_api
 
     def get_client(self) -> httpx.Client:
         return httpx.Client(
@@ -147,7 +180,9 @@ class Windmill:
             timeout = timeout.total_seconds()
 
         job_id = self.run_script_async(path=path, hash_=hash_, args=args)
-        return self.wait_job(job_id, timeout, verbose, cleanup, assert_result_is_not_none)
+        return self.wait_job(
+            job_id, timeout, verbose, cleanup, assert_result_is_not_none
+        )
 
     def wait_job(
         self,
@@ -173,7 +208,9 @@ class Windmill:
             timeout = timeout.total_seconds()
 
         while True:
-            result_res = self.get(f"/w/{self.workspace}/jobs_u/completed/get_result_maybe/{job_id}", True).json()
+            result_res = self.get(
+                f"/w/{self.workspace}/jobs_u/completed/get_result_maybe/{job_id}", True
+            ).json()
 
             started = result_res["started"]
             completed = result_res["completed"]
@@ -276,13 +313,31 @@ class Windmill:
             return result_text
 
     def get_variable(self, path: str) -> str:
+        path = parse_variable_syntax(path) or path
+        if self.mocked_api is not None:
+            variables = self.mocked_api["variables"]
+            try:
+                result = variables[path]
+                return result
+            except KeyError:
+                logger.info(
+                    f"MockedAPI present, but variable not found at {path}, falling back to real API"
+                )
+
         """Get variable from Windmill"""
         return self.get(f"/w/{self.workspace}/variables/get_value/{path}").json()
 
     def set_variable(self, path: str, value: str, is_secret: bool = False) -> None:
+        path = parse_variable_syntax(path) or path
+        if self.mocked_api is not None:
+            self.mocked_api["variables"][path] = value
+            return
+
         """Set variable from Windmill"""
         # check if variable exists
-        r = self.get(f"/w/{self.workspace}/variables/get/{path}", raise_for_status=False)
+        r = self.get(
+            f"/w/{self.workspace}/variables/get/{path}", raise_for_status=False
+        )
         if r.status_code == 404:
             # create variable
             self.post(
@@ -306,9 +361,28 @@ class Windmill:
         path: str,
         none_if_undefined: bool = False,
     ) -> dict | None:
+        path = parse_resource_syntax(path) or path
+        if self.mocked_api is not None:
+            resources = self.mocked_api["resources"]
+            try:
+                result = resources[path]
+                return result
+            except KeyError:
+                # NOTE: should mocked_api respect `none_if_undefined`?
+                if none_if_undefined:
+                    logger.info(
+                        f"resource not found at ${path}, but none_if_undefined is True, so returning None"
+                    )
+                    return None
+                logger.info(
+                    f"MockedAPI present, but resource not found at ${path}, falling back to real API"
+                )
+
         """Get resource from Windmill"""
         try:
-            return self.get(f"/w/{self.workspace}/resources/get_value_interpolated/{path}").json()
+            return self.get(
+                f"/w/{self.workspace}/resources/get_value_interpolated/{path}"
+            ).json()
         except Exception as e:
             if none_if_undefined:
                 return None
@@ -321,8 +395,15 @@ class Windmill:
         path: str,
         resource_type: str,
     ):
+        path = parse_resource_syntax(path) or path
+        if self.mocked_api is not None:
+            self.mocked_api["resources"][path] = value
+            return
+
         # check if resource exists
-        r = self.get(f"/w/{self.workspace}/resources/get/{path}", raise_for_status=False)
+        r = self.get(
+            f"/w/{self.workspace}/resources/get/{path}", raise_for_status=False
+        )
         if r.status_code == 404:
             # create resource
             self.post(
@@ -358,15 +439,15 @@ class Windmill:
                 "percent": value,
                 "flow_job_id": flow_id or None,
             },
-        )        
+        )
 
-    def get_progress(self, job_id: Optional[str] = None ) -> Any:
+    def get_progress(self, job_id: Optional[str] = None) -> Any:
         workspace = get_workspace()
         job_id = job_id or os.environ.get("WM_JOB_ID")
 
         r = self.get(
             f"/w/{workspace}/job_metrics/get_progress/{job_id}",
-        )        
+        )
         if r.status_code == 404:
             print(f"Job {job_id} does not exist")
             return None
@@ -376,14 +457,21 @@ class Windmill:
     def set_flow_user_state(self, key: str, value: Any) -> None:
         """Set the user state of a flow at a given key"""
         flow_id = self.get_root_job_id()
-        r = self.post(f"/w/{self.workspace}/jobs/flow/user_states/{flow_id}/{key}", json=value, raise_for_status=False)
+        r = self.post(
+            f"/w/{self.workspace}/jobs/flow/user_states/{flow_id}/{key}",
+            json=value,
+            raise_for_status=False,
+        )
         if r.status_code == 404:
             print(f"Job {flow_id} does not exist or is not a flow")
 
     def get_flow_user_state(self, key: str) -> Any:
         """Get the user state of a flow at a given key"""
         flow_id = self.get_root_job_id()
-        r = self.get(f"/w/{self.workspace}/jobs/flow/user_states/{flow_id}/{key}", raise_for_status=False)
+        r = self.get(
+            f"/w/{self.workspace}/jobs/flow/user_states/{flow_id}/{key}",
+            raise_for_status=False,
+        )
         if r.status_code == 404:
             print(f"Job {flow_id} does not exist or is not a flow")
             return None
@@ -402,14 +490,19 @@ class Windmill:
         Convenient helpers that takes an S3 resource as input and returns the settings necessary to
         initiate an S3 connection from DuckDB
         """
+        s3_resource_path = parse_resource_syntax(s3_resource_path) or s3_resource_path
         try:
             raw_obj = self.post(
                 f"/w/{self.workspace}/job_helpers/v2/duckdb_connection_settings",
-                json={} if s3_resource_path == "" else {"s3_resource_path": s3_resource_path},
+                json={}
+                if s3_resource_path == ""
+                else {"s3_resource_path": s3_resource_path},
             ).json()
             return DuckDbConnectionSettings(raw_obj)
         except JSONDecodeError as e:
-            raise Exception("Could not generate DuckDB S3 connection settings from the provided resource") from e
+            raise Exception(
+                "Could not generate DuckDB S3 connection settings from the provided resource"
+            ) from e
 
     def get_polars_connection_settings(
         self,
@@ -419,14 +512,19 @@ class Windmill:
         Convenient helpers that takes an S3 resource as input and returns the settings necessary to
         initiate an S3 connection from Polars
         """
+        s3_resource_path = parse_resource_syntax(s3_resource_path) or s3_resource_path
         try:
             raw_obj = self.post(
                 f"/w/{self.workspace}/job_helpers/v2/polars_connection_settings",
-                json={} if s3_resource_path == "" else {"s3_resource_path": s3_resource_path},
+                json={}
+                if s3_resource_path == ""
+                else {"s3_resource_path": s3_resource_path},
             ).json()
             return PolarsConnectionSettings(raw_obj)
         except JSONDecodeError as e:
-            raise Exception("Could not generate Polars S3 connection settings from the provided resource") from e
+            raise Exception(
+                "Could not generate Polars S3 connection settings from the provided resource"
+            ) from e
 
     def get_boto3_connection_settings(
         self,
@@ -436,16 +534,21 @@ class Windmill:
         Convenient helpers that takes an S3 resource as input and returns the settings necessary to
         initiate an S3 connection using boto3
         """
+        s3_resource_path = parse_resource_syntax(s3_resource_path) or s3_resource_path
         try:
             s3_resource = self.post(
                 f"/w/{self.workspace}/job_helpers/v2/s3_resource_info",
-                json={} if s3_resource_path == "" else {"s3_resource_path": s3_resource_path},
+                json={}
+                if s3_resource_path == ""
+                else {"s3_resource_path": s3_resource_path},
             ).json()
             return self.__boto3_connection_settings(s3_resource)
         except JSONDecodeError as e:
-            raise Exception("Could not generate Boto3 S3 connection settings from the provided resource") from e
+            raise Exception(
+                "Could not generate Boto3 S3 connection settings from the provided resource"
+            ) from e
 
-    def load_s3_file(self, s3object: S3Object, s3_resource_path: str | None) -> bytes:
+    def load_s3_file(self, s3object: S3Object | str, s3_resource_path: str | None) -> bytes:
         """
         Load a file from the workspace s3 bucket and returns its content as bytes.
 
@@ -457,10 +560,13 @@ class Windmill:
         file_content = my_obj_content.decode("utf-8")
         '''
         """
+        s3object = parse_s3_object(s3object)
         with self.load_s3_file_reader(s3object, s3_resource_path) as file_reader:
             return file_reader.read()
 
-    def load_s3_file_reader(self, s3object: S3Object, s3_resource_path: str | None) -> BufferedReader:
+    def load_s3_file_reader(
+        self, s3object: S3Object | str, s3_resource_path: str | None
+    ) -> BufferedReader:
         """
         Load a file from the workspace s3 bucket and returns the bytes stream.
 
@@ -472,14 +578,23 @@ class Windmill:
             print(file_reader.read())
         '''
         """
-        reader = S3BufferedReader(f"{self.workspace}", self.client, s3object["s3"], s3_resource_path, s3object["storage"] if "storage" in s3object else None)
+        s3object = parse_s3_object(s3object)
+        reader = S3BufferedReader(
+            f"{self.workspace}",
+            self.client,
+            s3object["s3"],
+            s3_resource_path,
+            s3object["storage"] if "storage" in s3object else None,
+        )
         return reader
 
     def write_s3_file(
         self,
-        s3object: S3Object | None,
+        s3object: S3Object | str | None,
         file_content: BufferedReader | bytes,
         s3_resource_path: str | None,
+        content_type: str | None = None,
+        content_disposition: str | None = None,
     ) -> S3Object:
         """
         Write a file to the workspace S3 bucket
@@ -498,6 +613,7 @@ class Windmill:
             client.write_s3_file(s3_obj, my_file)
         '''
         """
+        s3object = parse_s3_object(s3object)
         # httpx accepts either bytes or "a bytes generator" as content. If it's a BufferedReader, we need to convert it to a generator
         if isinstance(file_content, BufferedReader):
             content_payload = bytes_generator(file_content)
@@ -511,14 +627,25 @@ class Windmill:
             query_params["file_key"] = s3object["s3"]
         if s3_resource_path is not None and s3_resource_path != "":
             query_params["s3_resource_path"] = s3_resource_path
-        if s3object is not None and  "storage" in s3object and s3object["storage"] is not None:
+        if (
+            s3object is not None
+            and "storage" in s3object
+            and s3object["storage"] is not None
+        ):
             query_params["storage"] = s3object["storage"]
+        if content_type is not None:
+            query_params["content_type"] = content_type
+        if content_disposition is not None:
+            query_params["content_disposition"] = content_disposition
 
         try:
             # need a vanilla client b/c content-type is not application/json here
             response = httpx.post(
                 f"{self.base_url}/w/{self.workspace}/job_helpers/upload_s3_file",
-                headers={"Authorization": f"Bearer {self.token}", "Content-Type": "application/octet-stream"},
+                headers={
+                    "Authorization": f"Bearer {self.token}",
+                    "Content-Type": "application/octet-stream",
+                },
                 params=query_params,
                 content=content_payload,
                 verify=self.verify,
@@ -528,11 +655,24 @@ class Windmill:
             raise Exception("Could not write file to S3") from e
         return S3Object(s3=response["file_key"])
 
+    def sign_s3_objects(self, s3_objects: list[S3Object | str]) -> list[S3Object]:
+        return self.post(
+            f"/w/{self.workspace}/apps/sign_s3_objects", json={"s3_objects": list(map(parse_s3_object, s3_objects))}
+        ).json()
+
+    def sign_s3_object(self, s3_object: S3Object | str) -> S3Object:
+        return self.post(
+            f"/w/{self.workspace}/apps/sign_s3_objects",
+            json={"s3_objects": [s3_object]},
+        ).json()[0]
+
     def __boto3_connection_settings(self, s3_resource) -> Boto3ConnectionSettings:
         endpoint_url_prefix = "https://" if s3_resource["useSSL"] else "http://"
         return Boto3ConnectionSettings(
             {
-                "endpoint_url": "{}{}".format(endpoint_url_prefix, s3_resource["endPoint"]),
+                "endpoint_url": "{}{}".format(
+                    endpoint_url_prefix, s3_resource["endPoint"]
+                ),
                 "region_name": s3_resource["region"],
                 "use_ssl": s3_resource["useSSL"],
                 "aws_access_key_id": s3_resource["accessKey"],
@@ -550,7 +690,9 @@ class Windmill:
 
     @property
     def state_path(self) -> str:
-        state_path = os.environ.get("WM_STATE_PATH_NEW", os.environ.get("WM_STATE_PATH"))
+        state_path = os.environ.get(
+            "WM_STATE_PATH_NEW", os.environ.get("WM_STATE_PATH")
+        )
         if state_path is None:
             raise Exception("State path not found")
         return state_path
@@ -611,6 +753,83 @@ class Windmill:
             params={"approver": approver},
         ).json()
 
+    def request_interactive_slack_approval(
+        self,
+        slack_resource_path: str,
+        channel_id: str,
+        message: str = None,
+        approver: str = None,
+        default_args_json: dict = None,
+        dynamic_enums_json: dict = None,
+    ) -> None:
+        """
+        Sends an interactive approval request via Slack, allowing optional customization of the message, approver, and form fields.
+
+        **[Enterprise Edition Only]** To include form fields in the Slack approval request, use the "Advanced -> Suspend -> Form" functionality.
+        Learn more at: https://www.windmill.dev/docs/flows/flow_approval#form
+
+        :param slack_resource_path: The path to the Slack resource in Windmill.
+        :type slack_resource_path: str
+        :param channel_id: The Slack channel ID where the approval request will be sent.
+        :type channel_id: str
+        :param message: Optional custom message to include in the Slack approval request.
+        :type message: str, optional
+        :param approver: Optional user ID or name of the approver for the request.
+        :type approver: str, optional
+        :param default_args_json: Optional dictionary defining or overriding the default arguments for form fields.
+        :type default_args_json: dict, optional
+        :param dynamic_enums_json: Optional dictionary overriding the enum default values of enum form fields.
+        :type dynamic_enums_json: dict, optional
+
+        :raises Exception: If the function is not called within a flow or flow preview.
+        :raises Exception: If the required flow job or flow step environment variables are not set.
+
+        :return: None
+
+        **Usage Example:**
+            >>> client.request_interactive_slack_approval(
+            ...     slack_resource_path="/u/alex/my_slack_resource",
+            ...     channel_id="admins-slack-channel",
+            ...     message="Please approve this request",
+            ...     approver="approver123",
+            ...     default_args_json={"key1": "value1", "key2": 42},
+            ...     dynamic_enums_json={"foo": ["choice1", "choice2"], "bar": ["optionA", "optionB"]},
+            ... )
+
+        **Notes:**
+        - This function must be executed within a Windmill flow or flow preview.
+        - The function checks for required environment variables (`WM_FLOW_JOB_ID`, `WM_FLOW_STEP_ID`) to ensure it is run in the appropriate context.
+        """
+        workspace = self.workspace
+        flow_job_id = os.environ.get("WM_FLOW_JOB_ID")
+
+        if not flow_job_id:
+            raise Exception(
+                "You can't use 'request_interactive_slack_approval' function in a standalone script or flow step preview. Please use it in a flow or a flow preview."
+            )
+
+        # Only include non-empty parameters
+        params = {}
+        if message:
+            params["message"] = message
+        if approver:
+            params["approver"] = approver
+        if slack_resource_path:
+            params["slack_resource_path"] = slack_resource_path
+        if channel_id:
+            params["channel_id"] = channel_id
+        if os.environ.get("WM_FLOW_STEP_ID"):
+            params["flow_step_id"] = os.environ.get("WM_FLOW_STEP_ID")
+        if default_args_json:
+            params["default_args_json"] = json.dumps(default_args_json)
+        if dynamic_enums_json:
+            params["dynamic_enums_json"] = json.dumps(dynamic_enums_json)
+
+        self.get(
+            f"/w/{workspace}/jobs/slack_approval/{os.environ.get('WM_JOB_ID', 'NO_JOB_ID')}",
+            params=params,
+        )
+
     def username_to_email(self, username: str) -> str:
         """
         Get email from workspace username
@@ -619,6 +838,26 @@ class Windmill:
         """
         return self.get(f"/w/{self.workspace}/users/username_to_email/{username}").text
 
+    def send_teams_message(
+        self,
+        conversation_id: str,
+        text: str,
+        success: bool = True,
+        card_block: dict = None,
+    ):
+        """
+        Send a message to a Microsoft Teams conversation with conversation_id, where success is used to style the message
+        """
+        return self.post(
+            f"/teams/activities",
+            json={
+                "conversation_id": conversation_id,
+                "text": text,
+                "success": success,
+                "card_block": card_block,
+            },
+        )
+
 
 def init_global_client(f):
     @functools.wraps(f)
@@ -766,7 +1005,9 @@ def get_job_status(job_id: str) -> JobStatus:
 
 @init_global_client
 def get_result(job_id: str, assert_result_is_not_none=True) -> Dict[str, Any]:
-    return _client.get_result(job_id=job_id, assert_result_is_not_none=assert_result_is_not_none)
+    return _client.get_result(
+        job_id=job_id, assert_result_is_not_none=assert_result_is_not_none
+    )
 
 
 @init_global_client
@@ -797,31 +1038,69 @@ def boto3_connection_settings(s3_resource_path: str = "") -> Boto3ConnectionSett
 
 
 @init_global_client
-def load_s3_file(s3object: S3Object, s3_resource_path: str | None = None) -> bytes:
+def load_s3_file(s3object: S3Object | str, s3_resource_path: str | None = None) -> bytes:
     """
     Load the entire content of a file stored in S3 as bytes
     """
-    return _client.load_s3_file(s3object, s3_resource_path if s3_resource_path != "" else None)
+    return _client.load_s3_file(
+        s3object, s3_resource_path if s3_resource_path != "" else None
+    )
 
 
 @init_global_client
-def load_s3_file_reader(s3object: S3Object, s3_resource_path: str | None = None) -> BufferedReader:
+def load_s3_file_reader(
+    s3object: S3Object | str, s3_resource_path: str | None = None
+) -> BufferedReader:
     """
     Load the content of a file stored in S3
     """
-    return _client.load_s3_file_reader(s3object, s3_resource_path if s3_resource_path != "" else None)
+    return _client.load_s3_file_reader(
+        s3object, s3_resource_path if s3_resource_path != "" else None
+    )
 
 
 @init_global_client
 def write_s3_file(
-    s3object: S3Object | None,
+    s3object: S3Object | str | None,
     file_content: BufferedReader | bytes,
     s3_resource_path: str | None = None,
+    content_type: str | None = None,
+    content_disposition: str | None = None,
 ) -> S3Object:
     """
     Upload a file to S3
+
+    Content type will be automatically guessed from path extension if left empty
+
+    See MDN for content_disposition: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Disposition
+    and content_type: https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Type
+
     """
-    return _client.write_s3_file(s3object, file_content, s3_resource_path if s3_resource_path != "" else None)
+    return _client.write_s3_file(
+        s3object,
+        file_content,
+        s3_resource_path if s3_resource_path != "" else None,
+        content_type,
+        content_disposition,
+    )
+
+
+@init_global_client
+def sign_s3_objects(s3_objects: list[S3Object | str]) -> list[S3Object]:
+    """
+    Sign S3 objects to be used by anonymous users in public apps
+    Returns a list of signed s3 tokens
+    """
+    return _client.sign_s3_objects(s3_objects)
+
+
+@init_global_client
+def sign_s3_object(s3_object: S3Object| str) -> S3Object:
+    """
+    Sign S3 object to be used by anonymous users in public apps
+    Returns a signed s3 object
+    """
+    return _client.sign_s3_object(s3_object)
 
 
 @init_global_client
@@ -865,6 +1144,7 @@ def set_state(value: Any) -> None:
     """
     return _client.set_state(value)
 
+
 @init_global_client
 def set_progress(value: int, job_id: Optional[str] = None) -> None:
     """
@@ -872,6 +1152,7 @@ def set_progress(value: int, job_id: Optional[str] = None) -> None:
     """
     return _client.set_progress(value, job_id)
 
+
 @init_global_client
 def get_progress(job_id: Optional[str] = None) -> Any:
     """
@@ -951,6 +1232,32 @@ def get_resume_urls(approver: str = None) -> dict:
     return _client.get_resume_urls(approver)
 
 
+@init_global_client
+def request_interactive_slack_approval(
+    slack_resource_path: str,
+    channel_id: str,
+    message: str = None,
+    approver: str = None,
+    default_args_json: dict = None,
+    dynamic_enums_json: dict = None,
+) -> None:
+    return _client.request_interactive_slack_approval(
+        slack_resource_path=slack_resource_path,
+        channel_id=channel_id,
+        message=message,
+        approver=approver,
+        default_args_json=default_args_json,
+        dynamic_enums_json=dynamic_enums_json,
+    )
+
+
+@init_global_client
+def send_teams_message(
+    conversation_id: str, text: str, success: bool, card_block: dict = None
+):
+    return _client.send_teams_message(conversation_id, text, success, card_block)
+
+
 @init_global_client
 def cancel_running() -> dict:
     """Cancel currently running executions of the same script."""
@@ -993,7 +1300,10 @@ def task(*args, **kwargs):
     from inspect import signature
 
     def f(func, tag: str | None = None):
-        if os.environ.get("WM_JOB_ID") is None or os.environ.get("MAIN_OVERRIDE") == func.__name__:
+        if (
+            os.environ.get("WM_JOB_ID") is None
+            or os.environ.get("MAIN_OVERRIDE") == func.__name__
+        ):
 
             def inner(*args, **kwargs):
                 return func(*args, **kwargs)
@@ -1037,3 +1347,31 @@ def task(*args, **kwargs):
         return f(args[0], None)
     else:
         return lambda x: f(x, kwargs.get("tag"))
+
+def parse_resource_syntax(s: str) -> Optional[str]:
+    """Parse resource syntax from string."""
+    if s is None:
+        return None
+    if s.startswith("$res:"):
+        return s[5:]
+    if s.startswith("res://"):
+        return s[6:]
+    return None
+
+def parse_s3_object(s3_object: S3Object | str) -> S3Object:
+    """Parse S3 object from string or S3Object format."""
+    if isinstance(s3_object, str):
+        match = re.match(r'^s3://([^/]*)/(.*)$', s3_object)
+        if match:
+            return S3Object(s3=match.group(2) or "", storage=match.group(1) or None)
+        return S3Object(s3="")
+    else:
+        return s3_object
+
+    
+
+def parse_variable_syntax(s: str) -> Optional[str]:
+    """Parse variable syntax from string."""
+    if s.startswith("var://"):
+        return s[6:]
+    return None
\ No newline at end of file
